@@ -40,6 +40,7 @@ extern "C" {
 	#include <osmocom/core/stats.h>
 	#include <osmocom/gsm/protocol/gsm_04_08.h>
 	#include <osmocom/gsm/gsm_utils.h>
+	#include <osmocom/gsm/gsm0502.h>
 	#include <osmocom/gsm/gsm48.h>
 	#include <osmocom/core/gsmtap_util.h>
 	#include <osmocom/core/application.h>
@@ -245,6 +246,8 @@ static int bts_talloc_destructor(struct gprs_rlcmac_bts* bts)
 	bts->ms_store->cleanup();
 	delete bts->ms_store;
 
+	gprs_bssgp_destroy(bts);
+
 	osmo_time_cc_cleanup(&bts->all_allocated_pdch);
 
 	if (bts->ratectrs) {
@@ -335,13 +338,17 @@ struct gprs_rlcmac_bts* bts_alloc(struct gprs_pcu *pcu, uint8_t bts_nr)
 
 void bts_set_current_frame_number(struct gprs_rlcmac_bts *bts, uint32_t fn)
 {
+	/* See also 3GPP TS 45.002, section 4.3.3 */
+	OSMO_ASSERT(fn < GSM_TDMA_HYPERFRAME);
+
 	/* The UL frame numbers lag 3 behind the DL frames and the data
 	 * indication is only sent after all 4 frames of the block have been
 	 * received. Sometimes there is an idle frame between the end of one
 	 * and start of another frame (every 3 blocks). */
 	if (fn != bts->cur_fn && bts->cur_fn != FN_UNSET && fn != fn_next_block(bts->cur_fn)) {
 		LOGP(DRLCMAC, LOGL_NOTICE,
-			"Detected FN jump! %u -> %u\n", bts->cur_fn, fn);
+		     "Detected FN jump! %u -> %u (expected %u, delta %u)\n",
+		     bts->cur_fn, fn, fn_next_block(bts->cur_fn), GSM_TDMA_FN_DIFF(bts->cur_fn, fn));
 	}
 	bts->cur_fn = fn;
 }
@@ -420,7 +427,7 @@ int bts_add_paging(struct gprs_rlcmac_bts *bts, const struct paging_req_cs *req,
 	struct gprs_rlcmac_tbf *tbf;
 	struct llist_head *tmp;
 	const struct osmo_mobile_identity *mi;
-	uint8_t slot_mask[8];
+	uint8_t slot_mask[ARRAY_SIZE(bts->trx)];
 	int8_t first_ts; /* must be signed */
 
 	/* First, build the MI used to page on PDCH from available subscriber info: */
@@ -457,7 +464,7 @@ int bts_add_paging(struct gprs_rlcmac_bts *bts, const struct paging_req_cs *req,
 		ms = llist_entry(tmp, typeof(*ms), list);
 		struct gprs_rlcmac_tbf *tbfs[] = { ms->ul_tbf, ms->dl_tbf };
 		for (l = 0; l < ARRAY_SIZE(tbfs); l++) {
-			tbf = (struct gprs_rlcmac_tbf *)tbfs[l];
+			tbf = tbfs[l];
 			if (!tbf)
 				continue;
 			first_ts = -1;
@@ -628,7 +635,7 @@ int bts_tfi_find_free(const struct gprs_rlcmac_bts *bts, enum gprs_rlcmac_tbf_di
 	unsigned int best_cnt = 0;
 	uint8_t best_first_tfi = 0;
 
-	if (use_trx >= 0 && use_trx < 8)
+	if (use_trx >= 0 && use_trx < (int8_t)ARRAY_SIZE(bts->trx))
 		trx_from = trx_to = use_trx;
 	else {
 		trx_from = 0;
@@ -663,19 +670,24 @@ int bts_tfi_find_free(const struct gprs_rlcmac_bts *bts, enum gprs_rlcmac_tbf_di
 
 int bts_rcv_imm_ass_cnf(struct gprs_rlcmac_bts *bts, const uint8_t *data, uint32_t fn)
 {
-	struct gprs_rlcmac_dl_tbf *dl_tbf = NULL;
+	struct gprs_rlcmac_dl_tbf *dl_tbf;
+	const struct gsm48_imm_ass *imm_ass = (struct gsm48_imm_ass *)data;
 	uint8_t plen;
 	uint32_t tlli;
 	GprsMs *ms;
 
-	/* move to IA Rest Octets */
-	plen = data[0] >> 2;
+	/* Move to IA Rest Octets: TS 44.018 9.1.18 "The L2 pseudo length of
+	 * this message is the sum of lengths of all information elements
+	 * present in the message except the IA Rest Octets and L2 Pseudo Length
+	 * information elements." */
+	/* TS 44.018 10.5.2.19 l2_plen byte lowest 2 bits are '01'B */
+	plen = imm_ass->l2_plen >> 2;
 	data += 1 + plen;
 
 	if ((*data & 0xf0) != 0xd0) {
-		LOGP(DRLCMAC, LOGL_ERROR, "Got IMM.ASS confirm, but rest "
+		LOGP(DTBFDL, LOGL_ERROR, "FN=%u Got IMM.ASS confirm, but rest "
 			"octets do not start with bit sequence 'HH01' "
-			"(Packet Downlink Assignment)\n");
+			"(Packet Downlink Assignment)\n", fn);
 		return -EINVAL;
 	}
 
@@ -687,30 +699,31 @@ int bts_rcv_imm_ass_cnf(struct gprs_rlcmac_bts *bts, const uint8_t *data, uint32
 	tlli |= (*data++) >> 4;
 
 	ms = bts_ms_by_tlli(bts, tlli, GSM_RESERVED_TMSI);
-	if (ms)
-		dl_tbf = ms_dl_tbf(ms);
+	if (!ms) {
+		LOGP(DTBFDL, LOGL_ERROR, "FN=%u Got IMM.ASS confirm for unknown MS with TLLI=%08x\n", fn, tlli);
+		return -EINVAL;
+	}
+	dl_tbf = ms_dl_tbf(ms);
 	if (!dl_tbf) {
-		LOGP(DRLCMAC, LOGL_ERROR, "Got IMM.ASS confirm, but TLLI=%08x "
-			"does not exit\n", tlli);
+		LOGPMS(ms, DTBFDL, LOGL_ERROR, "FN=%u Got IMM.ASS confirm, but MS has no DL TBF!\n", fn);
 		return -EINVAL;
 	}
 
-	LOGP(DRLCMAC, LOGL_DEBUG, "Got IMM.ASS confirm for TLLI=%08x\n", tlli);
-	osmo_fsm_inst_dispatch(dl_tbf->state_fsm.fi, TBF_EV_ASSIGN_PCUIF_CNF, NULL);
+	LOGPTBFDL(dl_tbf, LOGL_DEBUG, "FN=%u Got IMM.ASS confirm\n", fn);
+	osmo_fsm_inst_dispatch(dl_tbf->state_fi, TBF_EV_ASSIGN_PCUIF_CNF, NULL);
 
 	return 0;
 }
 
 /* Determine the full frame number from a relative frame number */
-uint32_t bts_rfn_to_fn(const struct gprs_rlcmac_bts *bts, int32_t rfn)
+uint32_t bts_rfn_to_fn(const struct gprs_rlcmac_bts *bts, uint32_t rfn)
 {
-	int32_t m_cur_rfn;
-	int32_t fn;
-	int32_t fn_rounded;
+	uint32_t m_cur_rfn;
+	uint32_t fn_rounded;
 
-	/* double-check that relative FN is not negative and fits into int32_t */
+	/* make sure RFN does not exceed the maximum possible value of a valid
+	 * GSM frame number. */
 	OSMO_ASSERT(rfn < GSM_MAX_FN);
-	OSMO_ASSERT(rfn >= 0);
 
 	/* Note: If a BTS is sending in a rach request it will be fully aware
 	 * of the frame number. If the PCU is used in a BSC-co-located setup.
@@ -730,13 +743,13 @@ uint32_t bts_rfn_to_fn(const struct gprs_rlcmac_bts *bts, int32_t rfn)
 
 	/* Compute a "rounded" version of the internal frame number, which
 	 * exactly fits in the RFN_MODULUS raster */
-	fn_rounded = bts->cur_fn - m_cur_rfn;
+	fn_rounded = GSM_TDMA_FN_SUB(bts->cur_fn, m_cur_rfn);
 
 	/* If the delta between the internal and the external relative frame
 	 * number exceeds a certain limit, we need to assume that the incoming
 	 * rach request belongs to a the previous rfn period. To correct this,
 	 * we roll back the rounded frame number by one RFN_MODULUS */
-	if (abs(rfn - m_cur_rfn) > RFN_THRESHOLD) {
+	if (GSM_TDMA_FN_DIFF(rfn, m_cur_rfn) > RFN_THRESHOLD) {
 		LOGP(DRLCMAC, LOGL_DEBUG,
 		     "Race condition between rfn (%u) and m_cur_fn (%u) detected: rfn belongs to the previous modulus %u cycle, wrapping...\n",
 		     rfn, bts->cur_fn, RFN_MODULUS);
@@ -744,17 +757,15 @@ uint32_t bts_rfn_to_fn(const struct gprs_rlcmac_bts *bts, int32_t rfn)
 			LOGP(DRLCMAC, LOGL_DEBUG,
 			"Cornercase detected: wrapping crosses %u border\n",
 			GSM_MAX_FN);
-			fn_rounded = GSM_MAX_FN - (RFN_MODULUS - fn_rounded);
+			fn_rounded = GSM_TDMA_FN_SUB(GSM_MAX_FN, (GSM_TDMA_FN_SUB(RFN_MODULUS, fn_rounded)));
 		}
 		else
-			fn_rounded -= RFN_MODULUS;
+			fn_rounded = GSM_TDMA_FN_SUB(fn_rounded, RFN_MODULUS);
 	}
 
 	/* The real frame number is the sum of the rounded frame number and the
 	 * relative framenumber computed via RACH */
-	fn = fn_rounded + rfn;
-
-	return fn;
+	return GSM_TDMA_FN_SUM(fn_rounded, rfn);
 }
 
 /* 3GPP TS 44.060:
@@ -970,14 +981,15 @@ int bts_rcv_rach(struct gprs_rlcmac_bts *bts, const struct rach_ind_params *rip)
 		bts_do_rate_ctr_inc(bts, CTR_IMMEDIATE_ASSIGN_UL_TBF_TWO_PHASE);
 	} else {
 		GprsMs *ms = bts_alloc_ms(bts, 0, chan_req.egprs_mslot_class);
-		tbf = tbf_alloc_ul_ccch(bts, ms);
+		tbf = ms_new_ul_tbf_assigned_agch(ms);
 		if (!tbf) {
 			/* Send RR Immediate Assignment Reject */
 			rc = -EBUSY;
 			goto send_imm_ass_rej;
 		}
 		tbf->set_ta(ta);
-		pdch = &tbf->trx->pdch[tbf->first_ts];
+		/* Only single TS can be allocated through AGCH, hence first TS is the only one: */
+		pdch = tbf_get_first_ts(tbf);
 		usf = tbf->m_usf[pdch->ts_no];
 		bts_do_rate_ctr_inc(bts, CTR_IMMEDIATE_ASSIGN_UL_TBF_ONE_PHASE);
 	}
@@ -1063,23 +1075,29 @@ int bts_rcv_ptcch_rach(struct gprs_rlcmac_bts *bts, const struct rach_ind_params
 	return 0;
 }
 
-void bts_snd_dl_ass(struct gprs_rlcmac_bts *bts, struct gprs_rlcmac_tbf *tbf)
+void bts_snd_dl_ass(struct gprs_rlcmac_bts *bts, const struct gprs_rlcmac_dl_tbf *tbf)
 {
-	uint8_t trx_no = tbf->trx->trx_no;
-	uint8_t ts_no = tbf->first_ts;
 	uint16_t pgroup = ms_paging_group(tbf_ms(tbf));
 	int plen;
+	const struct gprs_rlcmac_pdch *pdch;
 
 	LOGPTBF(tbf, LOGL_INFO, "TX: START Immediate Assignment Downlink (PCH)\n");
+
+	/* Only one TS can be assigned through PCH, hence the first one is the only one: */
+	pdch = tbf_get_first_ts_const(tbf);
+
 	bitvec *immediate_assignment = bitvec_alloc(22, tall_pcu_ctx); /* without plen */
 	bitvec_unhex(immediate_assignment, DUMMY_VEC); /* standard '2B'O padding */
-	/* use request reference that has maximum distance to current time,
-	 * so the assignment will not conflict with possible RACH requests. */
+	/* 3GPP TS 44.018, section 9.1.18.0d states that the network shall code the
+	 * Request Reference IE, e.g. by using a suitably offset frame number, such
+	 * that the resource reference cannot be confused with any CHANNEL REQUEST
+	 * message sent by a mobile station.  Use last_rts_fn + 21216 (16 TDMA
+	 * super-frame periods, or ~21.3 seconds) to achieve a decent distance. */
 	LOGP(DRLCMAC, LOGL_DEBUG, " - TRX=%d (%d) TS=%d TA=%d\n",
-		trx_no, tbf->trx->arfcn, ts_no, tbf->ta());
-	plen = Encoding::write_immediate_assignment(&bts->trx[trx_no].pdch[ts_no],
+	     tbf->trx->trx_no, tbf->trx->arfcn, pdch->ts_no, tbf->ta());
+	plen = Encoding::write_immediate_assignment(pdch,
 						    tbf, immediate_assignment, true, 125,
-						    (tbf->pdch[ts_no]->last_rts_fn + 21216) % GSM_MAX_FN,
+						    GSM_TDMA_FN_SUM(pdch->last_rts_fn, 21216),
 						    tbf->ta(), 7, false, 0,
 						    bts_get_ms_pwr_alpha(bts), bts->pcu->vty.gamma, -1,
 						    GSM_L1_BURST_TYPE_ACCESS_0);
@@ -1220,7 +1238,7 @@ void bts_update_tbf_ta(struct gprs_rlcmac_bts *bts, const char *p, uint32_t fn,
 	    poll->tbf_poll.poll_tbf->direction != GPRS_RLCMAC_UL_TBF)
 		goto no_tbf;
 
-	tbf = as_ul_tbf(poll->tbf_poll.poll_tbf);
+	tbf = tbf_as_ul_tbf(poll->tbf_poll.poll_tbf);
 	/* we need to distinguish TA information provided by L1
 	 * from PH-DATA-IND and PHY-RA-IND so that we can properly
 	 * update TA for given TBF
@@ -1379,7 +1397,7 @@ void bts_recalc_max_mcs(struct gprs_rlcmac_bts *bts)
 
 struct GprsMs *bts_ms_by_imsi(struct gprs_rlcmac_bts *bts, const char *imsi)
 {
-	return bts_ms_store(bts)->get_ms(0, 0, imsi);
+	return bts_ms_store(bts)->get_ms(GSM_RESERVED_TMSI, GSM_RESERVED_TMSI, imsi);
 }
 
 const struct llist_head* bts_ms_list(struct gprs_rlcmac_bts *bts)
