@@ -435,7 +435,7 @@ int Encoding::write_immediate_assignment_reject(bitvec *dest, uint16_t ra,
  */
 int Encoding::write_immediate_assignment(
 	const struct gprs_rlcmac_pdch *pdch,
-	struct gprs_rlcmac_tbf *tbf,
+	const struct gprs_rlcmac_tbf *tbf,
 	bitvec * dest, bool downlink, uint16_t ra,
 	uint32_t ref_fn, uint8_t ta,
 	uint8_t usf, bool polling, uint32_t fn, uint8_t alpha,
@@ -445,17 +445,17 @@ int Encoding::write_immediate_assignment(
 	int plen;
 	int rc;
 
-	bitvec_write_field(dest, &wp,0x0,4);  // Skip Indicator
-	bitvec_write_field(dest, &wp,0x6,4);  // Protocol Discriminator
-	bitvec_write_field(dest, &wp,0x3F,8); // Immediate Assignment Message Type
+	bitvec_write_field(dest, &wp, 0x0, 4);  // Skip Indicator
+	bitvec_write_field(dest, &wp, 0x6, 4);  // Protocol Discriminator
+	bitvec_write_field(dest, &wp, GSM48_MT_RR_IMM_ASS, 8); // Immediate Assignment Message Type
 
 	// 10.5.2.25b Dedicated mode or TBF
-	bitvec_write_field(dest, &wp,0x0,1);      // spare
-	bitvec_write_field(dest, &wp,0x0,1);      // TMA : Two-message assignment: No meaning
-	bitvec_write_field(dest, &wp,downlink,1); // Downlink : Downlink assignment to mobile in packet idle mode
-	bitvec_write_field(dest, &wp,0x1,1);      // T/D : TBF or dedicated mode: this message assigns a Temporary Block Flow (TBF).
+	bitvec_write_field(dest, &wp, 0x0, 1);      // spare
+	bitvec_write_field(dest, &wp, 0x0, 1);      // TMA : Two-message assignment: No meaning
+	bitvec_write_field(dest, &wp, downlink, 1); // Downlink : Downlink assignment to mobile in packet idle mode
+	bitvec_write_field(dest, &wp, 0x1, 1);      // T/D : TBF or dedicated mode: this message assigns a Temporary Block Flow (TBF).
 
-	bitvec_write_field(dest, &wp,0x0,4); // Page Mode
+	bitvec_write_field(dest, &wp, 0x0, 4); // Page Mode
 
 	// GSM 04.08 10.5.2.25a Packet Channel Description
 	bitvec_write_field(dest, &wp, 0x01, 5);			// Channel type
@@ -480,13 +480,13 @@ int Encoding::write_immediate_assignment(
 		bitvec_write_field(dest, &wp, ra, 8);	/* RACH value */
 	}
 
-	bitvec_write_field(dest, &wp,(ref_fn / (26 * 51)) % 32,5); // T1'
-	bitvec_write_field(dest, &wp,ref_fn % 51,6);               // T3
-	bitvec_write_field(dest, &wp,ref_fn % 26,5);               // T2
+	bitvec_write_field(dest, &wp, (ref_fn / (26 * 51)) % 32, 5); // T1'
+	bitvec_write_field(dest, &wp, ref_fn % 51, 6);               // T3
+	bitvec_write_field(dest, &wp, ref_fn % 26, 5);               // T2
 
 	// 10.5.2.40 Timing Advance
-	bitvec_write_field(dest, &wp,0x0,2); // spare
-	bitvec_write_field(dest, &wp,ta,6);  // Timing Advance value
+	bitvec_write_field(dest, &wp, 0x0, 2); // spare
+	bitvec_write_field(dest, &wp, ta, 6);  // Timing Advance value
 
 	/* 10.5.2.21 Mobile Allocation */
 	if (pdch->fh.enabled) {
@@ -506,9 +506,9 @@ int Encoding::write_immediate_assignment(
 	/* 3GPP TS 44.018 §10.5.2.16 IA Rest Octets */
 	dest->cur_bit = wp;
 	if (downlink) {
-		OSMO_ASSERT(as_dl_tbf(tbf) != NULL);
+		OSMO_ASSERT(tbf_as_dl_tbf_const(tbf) != NULL);
 
-		rc = write_ia_rest_downlink(as_dl_tbf(tbf), dest, polling, gsm48_ta_is_valid(ta), fn, alpha, gamma,
+		rc = write_ia_rest_downlink(tbf_as_dl_tbf_const(tbf), dest, polling, gsm48_ta_is_valid(ta), fn, alpha, gamma,
 					    ta_idx);
 	} else if (((burst_type == GSM_L1_BURST_TYPE_ACCESS_1) || (burst_type == GSM_L1_BURST_TYPE_ACCESS_2))) {
 		SET_L(dest); SET_H(dest); // "LH"
@@ -518,8 +518,8 @@ int Encoding::write_immediate_assignment(
 
 		SET_0(dest); // No < Access Technologies Request struct >
 
-		if (as_ul_tbf(tbf) != NULL)
-			rc = write_ia_rest_egprs_uplink_sba(as_ul_tbf(tbf), dest, usf, alpha, gamma, ta_idx);
+		if (tbf_as_ul_tbf_const(tbf) != NULL)
+			rc = write_ia_rest_egprs_uplink_sba(tbf_as_ul_tbf_const(tbf), dest, usf, alpha, gamma, ta_idx);
 		else
 			rc = write_ia_rest_egprs_uplink_mba(dest, fn, alpha, gamma);
 	} else {
@@ -528,8 +528,8 @@ int Encoding::write_immediate_assignment(
 		SET_H(dest); SET_H(dest); // "HH"
 		SET_0(dest); SET_0(dest); // "00" < Packet Uplink Assignment >
 
-		if (as_ul_tbf(tbf) != NULL)
-			rc = write_ia_rest_uplink_mba(as_ul_tbf(tbf), dest, usf, alpha, gamma, ta_idx);
+		if (tbf_as_ul_tbf_const(tbf) != NULL)
+			rc = write_ia_rest_uplink_mba(tbf_as_ul_tbf_const(tbf), dest, usf, alpha, gamma, ta_idx);
 		else
 			rc = write_ia_rest_uplink_sba(dest, fn, alpha, gamma);
 	}
@@ -553,7 +553,7 @@ static void gen_freq_params(Frequency_Parameters_t *freq_params,
 	Direct_encoding_1_t fh_params;
 
 	/* Check one PDCH, if it's hopping then all other should too */
-	pdch = tbf->pdch[tbf->first_ts];
+	pdch = tbf_get_first_ts_const(tbf);
 	OSMO_ASSERT(pdch != NULL);
 
 	/* Training Sequence Code */
@@ -562,7 +562,7 @@ static void gen_freq_params(Frequency_Parameters_t *freq_params,
 	/* If frequency hopping is not in use, encode a single ARFCN */
 	if (!pdch->fh.enabled) {
 		freq_params->UnionType = 0x00;
-		freq_params->u.ARFCN = tbf->trx->arfcn;
+		freq_params->u.ARFCN = pdch->trx->arfcn;
 		return;
 	}
 
@@ -573,6 +573,8 @@ static void gen_freq_params(Frequency_Parameters_t *freq_params,
 	fh_params.MAIO = pdch->fh.maio;
 	fh_params.GPRS_Mobile_Allocation.HSN = pdch->fh.hsn;
 	fh_params.GPRS_Mobile_Allocation.ElementsOf_RFL_NUMBER = 0;
+	memset(&fh_params.GPRS_Mobile_Allocation.RFL_NUMBER[0], 0x00,
+	       sizeof(fh_params.GPRS_Mobile_Allocation.RFL_NUMBER));
 
 	/* Mobile Allocation bitmap */
 	fh_params.GPRS_Mobile_Allocation.UnionType = 0; /* MA bitmap */
@@ -603,7 +605,7 @@ void write_packet_uplink_assignment(RlcMacDownlink_t *block, uint8_t old_tfi,
 
 	/* See 3GPP TS 44.060, section 11.2.29 */
 	pua = &block->u.Packet_Uplink_Assignment;
-	pua->MESSAGE_TYPE = 0x0a;  // Packet Uplink Assignment
+	pua->MESSAGE_TYPE = MT_PACKET_UPLINK_ASSIGNMENT;  // Packet Uplink Assignment
 	pua->PAGE_MODE    = 0x00;  // Normal Paging
 
 	/* TLLI or Global (UL/DL) TFI */
