@@ -12,10 +12,6 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include <gprs_rlcmac.h>
@@ -40,6 +36,7 @@
 #include "llc.h"
 #include "gprs_rlcmac.h"
 #include "bts_pch_timer.h"
+#include "alloc_algo.h"
 
 /* Tuning parameters for BSSGP flow control */
 #define FC_DEFAULT_LIFE_TIME_SECS 10		/* experimental value, 10s */
@@ -786,7 +783,7 @@ static unsigned count_pdch(const struct gprs_rlcmac_bts *bts)
 		for (ts_no = 0; ts_no < ARRAY_SIZE(trx->pdch); ++ts_no) {
 			const struct gprs_rlcmac_pdch *pdch = &trx->pdch[ts_no];
 
-			if (pdch->m_is_enabled)
+			if (pdch_is_enabled(pdch))
 				num_pdch += 1;
 		}
 	}
