@@ -12,10 +12,6 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 
@@ -32,8 +28,7 @@ extern "C" {
 static void ms_storage_ms_idle_cb(struct GprsMs *ms)
 {
 	llist_del(&ms->list);
-	if (ms->bts)
-		bts_stat_item_add(ms->bts, STAT_MS_PRESENT, -1);
+	bts_stat_item_dec(ms->bts, STAT_MS_PRESENT);
 	if (ms_is_idle(ms))
 		talloc_free(ms);
 }
@@ -66,7 +61,10 @@ void GprsMsStorage::cleanup()
 	llist_for_each_safe(pos, tmp, &m_list) {
 		struct GprsMs *ms = llist_entry(pos, typeof(*ms), list);
 		ms_set_callback(ms, NULL);
-		ms_storage_ms_idle_cb(ms);
+		ms_set_timeout(ms, 0);
+		llist_del(&ms->list);
+		bts_stat_item_dec(ms->bts, STAT_MS_PRESENT);
+		talloc_free(ms);
 	}
 }
 
@@ -107,7 +105,12 @@ GprsMs *GprsMsStorage::create_ms()
 	ms_set_callback(ms, &ms_storage_ms_cb);
 	llist_add(&ms->list, &m_list);
 	if (m_bts)
-		bts_stat_item_add(m_bts, STAT_MS_PRESENT, 1);
+		bts_stat_item_inc(m_bts, STAT_MS_PRESENT);
 
 	return ms;
 }
+
+struct GprsMs *ms_store_get_ms(struct GprsMsStorage *ms_store, uint32_t tlli, uint32_t old_tlli, const char *imsi)
+{
+	return ms_store->get_ms(tlli, old_tlli, imsi);
+}
