@@ -12,10 +12,6 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #pragma once
@@ -27,6 +23,17 @@
 
 struct gprs_rlcmac_bts;
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+struct GprsMsStorage;
+struct GprsMs *ms_store_get_ms(struct GprsMsStorage *ms_store, uint32_t tlli, uint32_t old_tlli, const char *imsi);
+#ifdef __cplusplus
+}
+#endif
+
+#ifdef __cplusplus
+
 struct GprsMsStorage {
 public:
 	GprsMsStorage(struct gprs_rlcmac_bts *bts);
@@ -42,3 +49,5 @@ private:
 	struct gprs_rlcmac_bts *m_bts;
 	struct llist_head m_list; /* list of struct GprsMs */
 };
+
+#endif
