@@ -23,6 +23,7 @@
 #include "bts.h"
 #include "tbf.h"
 #include "pcu_vty_functions.h"
+#include "alloc_algo.h"
 
 #define X(x) (1 << x)
 
@@ -1241,8 +1242,8 @@ DEFUN(show_ms_imsi,
 }
 
 static const char pcu_copyright[] =
-	"Copyright (C) 2012 by Ivan Kluchnikov <kluchnikovi@gmail.com> and \r\n"
-	"                      Andreas Eversberg <jolly@eversberg.eu>\r\n"
+	"Copyright (C) 2012-2013 by Ivan Kluchnikov and Andreas Eversberg\r\n"
+	"Copyright (C) 2013-2022 by sysmocom - s.f.m.c. GmbH\r\n"
 	"License GNU GPL version 2 or later\r\n"
 	"This is free software: you are free to change and redistribute it.\r\n"
 	"There is NO WARRANTY, to the extent permitted by law.\r\n";
