@@ -21,6 +21,14 @@
 
 #include <stdint.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+#include <tbf_fsm.h>
+#ifdef __cplusplus
+}
+#endif
+
 /*
  * TBF instance
  */
@@ -39,12 +47,9 @@ struct gprs_rlcmac_dl_tbf : public gprs_rlcmac_tbf {
 	~gprs_rlcmac_dl_tbf();
 	gprs_rlc_window *window();
 
-	int append_data(uint16_t pdu_delay_csec,
-			const uint8_t *data, uint16_t len);
-
 	int rcvd_dl_ack(bool final_ack, unsigned first_bsn, struct bitvec *rbb);
-	struct msgb *create_dl_acked_block(uint32_t fn, uint8_t ts, enum mcs_kind req_mcs_kind = EGPRS);
-	void trigger_ass(struct gprs_rlcmac_tbf *old_tbf);
+	struct msgb *create_dl_acked_block(uint32_t fn, const gprs_rlcmac_pdch *pdch,
+					   enum mcs_kind req_mcs_kind = EGPRS);
 
 	void request_dl_ack();
 	bool need_poll_for_dl_ack_nack() const;
@@ -82,6 +87,8 @@ struct gprs_rlcmac_dl_tbf : public gprs_rlcmac_tbf {
 	struct rate_ctr_group *m_dl_gprs_ctrs;
 	struct rate_ctr_group *m_dl_egprs_ctrs;
 
+	struct tbf_dl_fsm_ctx state_fsm;
+
 protected:
 	struct ana_result {
 		unsigned received_packets;
@@ -94,13 +101,12 @@ protected:
 			  bool *may_combine);
 	bool restart_bsn_cycle();
 	int create_new_bsn(const uint32_t fn, enum CodingScheme cs);
-	struct msgb *create_dl_acked_block(const uint32_t fn, const uint8_t ts,
-					int index, int index2 = -1);
+	struct msgb *create_dl_acked_block(const uint32_t fn, const struct gprs_rlcmac_pdch *pdch,
+					   int index, int index2 = -1);
 	int update_window(unsigned first_bsn, const struct bitvec *rbb);
 	int rcvd_dl_final_ack();
 	bool dl_window_stalled() const;
 	void reuse_tbf();
-	void start_llc_timer();
 	int analyse_errors(char *show_rbb, uint8_t ssn, ana_result *res);
 	void schedule_next_frame();
 
@@ -109,8 +115,6 @@ protected:
 	unsigned int get_egprs_dl_spb_status(int bsn);
 	enum egprs_rlcmac_dl_spb get_egprs_dl_spb(int bsn);
 
-	struct osmo_timer_list m_llc_timer;
-
 	/* Please note that all variables below will be reset when changing
 	 * from WAIT RELEASE back to FLOW state (re-use of TBF).
 	 * All states that need reset must be in this struct, so this is why
@@ -124,9 +128,6 @@ inline uint16_t gprs_rlcmac_dl_tbf::window_size() const
 	return m_window.ws();
 }
 
-struct gprs_rlcmac_dl_tbf *tbf_alloc_dl_tbf(struct gprs_rlcmac_bts *bts, GprsMs *ms,
-					    int8_t use_trx, bool single_slot);
-
 #else /* ifdef __cplusplus */
 struct gprs_rlcmac_dl_tbf;
 #endif
@@ -136,7 +137,11 @@ extern "C" {
 #endif
 struct gprs_rlcmac_bts;
 
-struct gprs_rlcmac_dl_tbf *as_dl_tbf(struct gprs_rlcmac_tbf *tbf);
+struct gprs_rlcmac_dl_tbf *dl_tbf_alloc(struct gprs_rlcmac_bts *bts, struct GprsMs *ms,
+					int8_t use_trx, bool single_slot);
+
+struct gprs_rlcmac_dl_tbf *tbf_as_dl_tbf(struct gprs_rlcmac_tbf *tbf);
+const struct gprs_rlcmac_dl_tbf *tbf_as_dl_tbf_const(const struct gprs_rlcmac_tbf *tbf);
 /* dispatch Unitdata.DL messages */
 int dl_tbf_handle(struct gprs_rlcmac_bts *bts,
 		  const uint32_t tlli, const uint32_t old_tlli,
@@ -144,9 +149,22 @@ int dl_tbf_handle(struct gprs_rlcmac_bts *bts,
 		  const uint8_t egprs_ms_class, const uint16_t delay_csec,
 		  const uint8_t *data, const uint16_t len);
 
-void tbf_dl_trigger_ass(struct gprs_rlcmac_dl_tbf *tbf, struct gprs_rlcmac_tbf *old_tbf);
+void dl_tbf_trigger_ass_on_pacch(struct gprs_rlcmac_dl_tbf *tbf, struct gprs_rlcmac_tbf *old_tbf);
+void dl_tbf_trigger_ass_on_pch(struct gprs_rlcmac_dl_tbf *tbf);
+void dl_tbf_request_dl_ack(struct gprs_rlcmac_dl_tbf *tbf);
+int dl_tbf_upgrade_to_multislot(struct gprs_rlcmac_dl_tbf *tbf);
 
-#define LOGPTBFDL(tbf, level, fmt, args...) LOGP(DTBFDL, level, "%s " fmt, tbf_name(tbf), ## args)
+static inline struct gprs_rlcmac_tbf *dl_tbf_as_tbf(struct gprs_rlcmac_dl_tbf *dl_tbf)
+{
+	return (struct gprs_rlcmac_tbf *)dl_tbf;
+}
+
+static inline const struct gprs_rlcmac_tbf *dl_tbf_as_tbf_const(const struct gprs_rlcmac_dl_tbf *dl_tbf)
+{
+	return (const struct gprs_rlcmac_tbf *)dl_tbf;
+}
+
+#define LOGPTBFDL(dl_tbf, level, fmt, args...) LOGP(DTBFDL, level, "%s " fmt, tbf_name(dl_tbf_as_tbf_const(dl_tbf)), ## args)
 #ifdef __cplusplus
 }
 #endif
