@@ -11,10 +11,6 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include "gprs_rlcmac.h"
@@ -31,6 +27,7 @@
 
 extern "C" {
 #include "mslot_class.h"
+#include "alloc_algo.h"
 #include <osmocom/core/application.h>
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/talloc.h>
@@ -50,9 +47,9 @@ static gprs_rlcmac_tbf *tbf_alloc(struct gprs_rlcmac_bts *bts,
 	OSMO_ASSERT(ms != NULL);
 
 	if (dir == GPRS_RLCMAC_UL_TBF)
-		return tbf_alloc_ul_tbf(bts, ms, use_trx, single_slot);
+		return ul_tbf_alloc(bts, ms, use_trx, single_slot);
 	else
-		return tbf_alloc_dl_tbf(bts, ms, use_trx, single_slot);
+		return dl_tbf_alloc(bts, ms, use_trx, single_slot);
 }
 
 static void check_tfi_usage(struct gprs_rlcmac_bts *bts)
@@ -180,12 +177,14 @@ static void dump_assignment(struct gprs_rlcmac_tbf *tbf, const char *dir, bool v
 {
 	if (!verbose)
 		return;
+	const struct GprsMs *ms = tbf_ms(tbf);
+	const struct gprs_rlcmac_pdch *first_common = ms_first_common_ts(ms);
 
 	for (size_t i = 0; i < ARRAY_SIZE(tbf->pdch); ++i)
 		if (tbf->pdch[i])
 			printf("PDCH[%zu] is used for %s\n", i, dir);
-	printf("PDCH[%d] is control_ts for %s\n", tbf->control_ts, dir);
-	printf("PDCH[%d] is first common for %s\n", tbf->first_common_ts, dir);
+	printf("%s is control_ts for %s\n", tbf->control_ts ? pdch_name(tbf->control_ts) : "(none)", dir);
+	printf("%s is first common for %s\n", first_common ? pdch_name(first_common) : "(none)", dir);
 }
 
 #define ENABLE_PDCH(ts_no, enable_flag, trx)	\
@@ -225,7 +224,7 @@ static inline bool test_alloc_b_ul_dl(bool ts0, bool ts1, bool ts2, bool ts3, bo
 	ms = bts_alloc_ms(bts, ms_class, 0);
 	/* Avoid delaying free to avoid tons of to-be-freed ms objects queuing */
 	ms_set_timeout(ms, 0);
-	ul_tbf = tbf_alloc_ul_tbf(bts, ms, -1, true);
+	ul_tbf = ul_tbf_alloc(bts, ms, -1, true);
 	if (!ul_tbf)
 		return false;
 
@@ -235,14 +234,12 @@ static inline bool test_alloc_b_ul_dl(bool ts0, bool ts1, bool ts2, bool ts3, bo
 	dump_assignment(ul_tbf, "UL", verbose);
 
 	/* assume final ack has not been sent */
-	dl_tbf = tbf_alloc_dl_tbf(bts, ms, ms_current_trx(ms)->trx_no, false);
+	dl_tbf = dl_tbf_alloc(bts, ms, ms_current_trx(ms)->trx_no, false);
 	if (!dl_tbf)
 		return false;
 
 	dump_assignment(dl_tbf, "DL", verbose);
 
-	OSMO_ASSERT(dl_tbf->first_common_ts == ul_tbf->first_common_ts);
-
 	check_tfi_usage(bts);
 
 	tbf_free(dl_tbf);
@@ -269,31 +266,28 @@ static inline bool test_alloc_b_dl_ul(bool ts0, bool ts1, bool ts2, bool ts3, bo
 	ms = bts_alloc_ms(bts, ms_class, 0);
 	/* Avoid delaying free to avoid tons of to-be-freed ms objects queuing */
 	ms_set_timeout(ms, 0);
-	dl_tbf = tbf_alloc_dl_tbf(bts, ms, -1, true);
+	dl_tbf = dl_tbf_alloc(bts, ms, -1, true);
 	if (!dl_tbf)
 		return false;
 
-	dl_tbf->update_ms(0x23, GPRS_RLCMAC_DL_TBF);
+	ms_confirm_tlli(ms, 0x23);
 	OSMO_ASSERT(dl_tbf->ms() == ms);
 	OSMO_ASSERT(ms_current_trx(dl_tbf->ms()));
 
 	dump_assignment(dl_tbf, "DL", verbose);
 
-	ul_tbf = tbf_alloc_ul_tbf(bts, ms, ms_current_trx(ms)->trx_no, false);
+	ul_tbf = ul_tbf_alloc(bts, ms, ms_current_trx(ms)->trx_no, false);
 	if (!ul_tbf)
 		return false;
 
-	ul_tbf->update_ms(0x23, GPRS_RLCMAC_UL_TBF);
-	ul_tbf->m_contention_resolution_done = 1;
+	ms_update_announced_tlli(ms, 0x23);
+	ul_tbf->m_contention_resolution_done = true;
 
 	dump_assignment(ul_tbf, "UL", verbose);
 
-	OSMO_ASSERT(dl_tbf->first_common_ts == ul_tbf->first_common_ts);
-
-	/* now update the dl_tbf */
-	dl_tbf->update();
+	/* now upgrade the dl_tbf */
+	OSMO_ASSERT(dl_tbf_upgrade_to_multislot(dl_tbf) == 0);
 	dump_assignment(dl_tbf, "DL", verbose);
-	OSMO_ASSERT(dl_tbf->first_common_ts == ul_tbf->first_common_ts);
 
 	check_tfi_usage(bts);
 
@@ -322,7 +316,7 @@ static inline bool test_alloc_b_jolly(uint8_t ms_class)
 	ms = bts_alloc_ms(bts, ms_class, 0);
 	/* Avoid delaying free to avoid tons of to-be-freed ms objects queuing */
 	ms_set_timeout(ms, 0);
-	ul_tbf = tbf_alloc_ul_tbf(bts, ms, -1, false);
+	ul_tbf = ul_tbf_alloc(bts, ms, -1, false);
 	if (!ul_tbf)
 		return false;
 
@@ -332,14 +326,12 @@ static inline bool test_alloc_b_jolly(uint8_t ms_class)
 	dump_assignment(ul_tbf, "UL", true);
 
 	/* assume final ack has not been sent */
-	dl_tbf = tbf_alloc_dl_tbf(bts, ms, trx_no, false);
+	dl_tbf = dl_tbf_alloc(bts, ms, trx_no, false);
 	if (!dl_tbf)
 		return false;
 
 	dump_assignment(dl_tbf, "DL", true);
 
-	OSMO_ASSERT(dl_tbf->first_common_ts == ul_tbf->first_common_ts);
-
 	check_tfi_usage(bts);
 
 	tbf_free(dl_tbf);
@@ -479,7 +471,7 @@ static GprsMs *alloc_tbfs(struct gprs_rlcmac_bts *bts, struct GprsMs *old_ms, en
 	case TEST_MODE_UL_AND_DL:
 		if (ms_ul_tbf(old_ms))
 			tbf_free(ms_ul_tbf(old_ms));
-		tbf = tbf_alloc_ul_tbf(bts, old_ms, trx_no, false);
+		tbf = ul_tbf_alloc(bts, old_ms, trx_no, false);
 		if (tbf == NULL) {
 			OSMO_ASSERT(trx_no != -1 || bts_all_pdch_allocated(bts));
 			ms_unref(old_ms);
@@ -491,7 +483,7 @@ static GprsMs *alloc_tbfs(struct gprs_rlcmac_bts *bts, struct GprsMs *old_ms, en
 	case TEST_MODE_DL_AND_UL:
 		if (ms_dl_tbf(old_ms))
 			tbf_free(ms_dl_tbf(old_ms));
-		tbf = tbf_alloc_dl_tbf(bts, old_ms, trx_no, false);
+		tbf = dl_tbf_alloc(bts, old_ms, trx_no, false);
 		if (tbf == NULL) {
 			OSMO_ASSERT(trx_no != -1 || bts_all_pdch_allocated(bts));
 			ms_unref(old_ms);
@@ -579,13 +571,13 @@ static unsigned alloc_many_tbfs(struct gprs_rlcmac_bts *bts, unsigned min_class,
 		trx = ms_current_trx(ms);
 
 		OSMO_ASSERT(ul_tbf || dl_tbf);
-
+		OSMO_ASSERT(ms_first_common_ts(ms) != NULL);
 		if (ul_tbf) {
-			ul_slots = 1 << ul_tbf->first_common_ts;
+			ul_slots = 1 << (uint8_t)ms_first_common_ts(ms)->ts_no;
 			tfi = ul_tbf->tfi();
 			dir = GPRS_RLCMAC_UL_TBF;
 		} else {
-			ul_slots = 1 << dl_tbf->first_common_ts;
+			ul_slots = 1 << (uint8_t)ms_first_common_ts(ms)->ts_no;
 			tfi = dl_tbf->tfi();
 			dir = GPRS_RLCMAC_DL_TBF;
 		}
@@ -776,7 +768,7 @@ static void test_2_consecutive_dl_tbfs()
 	trx->pdch[7].enable();
 
 	ms = bts_alloc_ms(bts, ms_class, egprs_ms_class);
-	dl_tbf1 = tbf_alloc_dl_tbf(bts, ms, 0, false);
+	dl_tbf1 = dl_tbf_alloc(bts, ms, 0, false);
 	OSMO_ASSERT(dl_tbf1);
 
 	for (int i = 0; i < 8; i++) {
@@ -787,7 +779,7 @@ static void test_2_consecutive_dl_tbfs()
 	printf("TBF1: numTs(%d)\n", numTs1);
 
 	ms = bts_alloc_ms(bts, ms_class, egprs_ms_class);
-	dl_tbf2 = tbf_alloc_dl_tbf(bts, ms, 0, false);
+	dl_tbf2 = dl_tbf_alloc(bts, ms, 0, false);
 	OSMO_ASSERT(dl_tbf2);
 
 	for (int i = 0; i < 8; i++) {
