@@ -11,29 +11,18 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 
 
-//#include <BitVector.h>
 #include <iostream>
-#include <cstdlib>
 #include <cstring>
-#include <assert.h>
 #include "gprs_rlcmac.h"
 #include "decoding.h"
 
 extern "C" {
-#include "pcu_vty.h"
-#include <osmocom/vty/telnet_interface.h>
-#include <osmocom/vty/logging.h>
 #include <osmocom/core/utils.h>
 #include <osmocom/core/msgb.h>
-#include <osmocom/core/application.h>
 
 #include "csn1.h"
 #include "gsm_rlcmac.h"
@@ -44,55 +33,7 @@ using namespace std;
 
 void *tall_pcu_ctx;
 
-void printSizeofRLCMAC()
-{
-	printf("*** %s ***\n", __func__);
-	cout << "sizeof RlcMacUplink_t                       " << sizeof(RlcMacUplink_t) << endl;
-	cout << "sizeof Packet_Cell_Change_Failure_t         " << sizeof(Packet_Cell_Change_Failure_t) << endl;
-	cout << "sizeof Packet_Control_Acknowledgement_t     " << sizeof(Packet_Control_Acknowledgement_t) << endl;
-	cout << "sizeof Packet_Downlink_Ack_Nack_t           " << sizeof(Packet_Downlink_Ack_Nack_t) << endl;
-	cout << "sizeof EGPRS_PD_AckNack_t		     " << sizeof(EGPRS_PD_AckNack_t) << endl;
-	cout << "sizeof Packet_Uplink_Dummy_Control_Block_t  " << sizeof(Packet_Uplink_Dummy_Control_Block_t) << endl;
-	cout << "sizeof Packet_Measurement_Report_t          " << sizeof(Packet_Measurement_Report_t) << endl;
-	cout << "sizeof Packet_Resource_Request_t            " << sizeof(Packet_Resource_Request_t) << endl;
-	cout << "sizeof Packet_Mobile_TBF_Status_t           " << sizeof(Packet_Mobile_TBF_Status_t) << endl;
-	cout << "sizeof Packet_PSI_Status_t                  " << sizeof(Packet_PSI_Status_t) << endl;
-	cout << "sizeof Packet_Enh_Measurement_Report_t      " << sizeof(Packet_Enh_Measurement_Report_t) << endl;
-	cout << "sizeof Packet_Cell_Change_Notification_t    " << sizeof(Packet_Cell_Change_Notification_t) << endl;
-	cout << "sizeof Packet_SI_Status_t                   " << sizeof(Packet_SI_Status_t) << endl;
-	cout << "sizeof Additional_MS_Rad_Access_Cap_t       " << sizeof(Additional_MS_Rad_Access_Cap_t) << endl;
-	cout << "sizeof Packet_Pause_t                       " << sizeof(Packet_Pause_t) << endl;
-
-	cout << "sizeof RlcMacDownlink_t                       " << sizeof(RlcMacDownlink_t) << endl;
-	cout << "sizeof Packet_Access_Reject_t                 " << sizeof(Packet_Access_Reject_t) << endl;
-	cout << "sizeof Packet_Cell_Change_Order_t             " << sizeof(Packet_Cell_Change_Order_t) << endl;
-	cout << "sizeof Packet_Downlink_Assignment_t           " << sizeof(Packet_Downlink_Assignment_t) << endl;
-	cout << "sizeof Packet_Neighbour_Cell_Data_t           " << sizeof(Packet_Neighbour_Cell_Data_t) << endl;
-	cout << "sizeof Packet_Serving_Cell_Data_t             " << sizeof(Packet_Serving_Cell_Data_t) << endl;
-	cout << "sizeof Packet_Paging_Request_t                " << sizeof(Packet_Paging_Request_t) << endl;
-	cout << "sizeof Packet_PDCH_Release_t                  " << sizeof(Packet_PDCH_Release_t) << endl;
-	cout << "sizeof Packet_Polling_Request_t               " << sizeof(Packet_Polling_Request_t) << endl;
-	cout << "sizeof Packet_Power_Control_Timing_Advance_t  " << sizeof(Packet_Power_Control_Timing_Advance_t) << endl;
-	cout << "sizeof Packet_PRACH_Parameters_t              " << sizeof(Packet_PRACH_Parameters_t) << endl;
-	cout << "sizeof Packet_Queueing_Notification_t         " << sizeof(Packet_Queueing_Notification_t) << endl;
-	cout << "sizeof Packet_Timeslot_Reconfigure_t          " << sizeof(Packet_Timeslot_Reconfigure_t) << endl;
-	cout << "sizeof Packet_TBF_Release_t                   " << sizeof(Packet_TBF_Release_t) << endl;
-	cout << "sizeof Packet_Uplink_Ack_Nack_t               " << sizeof(Packet_Uplink_Ack_Nack_t) << endl;
-	cout << "sizeof Packet_Uplink_Assignment_t             " << sizeof(Packet_Uplink_Assignment_t) << endl;
-	cout << "sizeof Packet_Cell_Change_Continue_t          " << sizeof(Packet_Cell_Change_Continue_t) << endl;
-	cout << "sizeof Packet_Handover_Command_t              " << sizeof(Packet_Handover_Command_t) << endl;
-	cout << "sizeof Packet_PhysicalInformation_t           " << sizeof(Packet_PhysicalInformation_t) << endl;
-	cout << "sizeof Packet_Downlink_Dummy_Control_Block_t  " << sizeof(Packet_Downlink_Dummy_Control_Block_t) << endl;
-	cout << "sizeof PSI1_t                " << sizeof(PSI1_t) << endl;
-	cout << "sizeof PSI2_t                " << sizeof(PSI2_t) << endl;
-	cout << "sizeof PSI3_t                " << sizeof(PSI3_t) << endl;
-	cout << "sizeof PSI3_BIS_t            " << sizeof(PSI3_BIS_t) << endl;
-	cout << "sizeof PSI4_t                " << sizeof(PSI4_t) << endl;
-	cout << "sizeof PSI13_t               " << sizeof(PSI13_t) << endl;
-	cout << "sizeof PSI5_t                " << sizeof(PSI5_t) << endl;
-}
-
-void testRlcMacDownlink(void *test_ctx)
+static void testRlcMacDownlink(void *test_ctx)
 {
 	printf("*** %s ***\n", __func__);
 
@@ -101,23 +42,21 @@ void testRlcMacDownlink(void *test_ctx)
 	bitvec_unhex(resultVector, DUMMY_VEC);
 
 	std::string testData[] = {
-	"4e082500e3f1a81d080820800b2b2b2b2b2b2b2b2b2b2b", // Packet Downlink Assignment
-	"48282407a6a07422720100032b2b2b2b2b2b2b2b2b2b2b", // Packet Uplink Assignment
-	"47240c00400000000000000079eb2ac9402b2b2b2b2b2b", // Packet Uplink Ack Nack
-	"47283c367513ba333004242b2b2b2b2b2b2b2b2b2b2b2b", // Packet Uplink Assignment
-	"400820001a3904df0680efb3300b2b2b2b2b2b2b2b2b2b", // Packet Downlink Assignment (EGPRS)
-	"40284f0000001009810c826f4406809dcecb2b2b2b2b2b", // Packet Uplink Assignment (EGPRS)
-	"4024030f2f0000000087b0042b2b2b2b2b2b2b2b2b2b2b", // Packet Uplink Ack Nack (EGPRS)
-	"4913e00850884013a8048b2b2b2b2b2b2b2b2b2b2b2b2b", // Polling Request (malformed)
-	"412430007fffffffffffffffefd19c7ba12b2b2b2b2b2b", // Packet Uplink Ack Nack?
-	"41942b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b", // System Info 13?
-	"40883c1493120000000012002b2b2b2b2b2b2b2b2b2b2b", // Pkt Paging Request (OS#4838)
+		"4e082500e3f1a81d080820800b2b2b2b2b2b2b2b2b2b2b", // Packet Downlink Assignment
+		"48282407a6a07422720100032b2b2b2b2b2b2b2b2b2b2b", // Packet Uplink Assignment
+		"47240c00400000000000000079eb2ac9402b2b2b2b2b2b", // Packet Uplink Ack Nack
+		"47283c367513ba333004242b2b2b2b2b2b2b2b2b2b2b2b", // Packet Uplink Assignment
+		"400820001a3904df0680efb3300b2b2b2b2b2b2b2b2b2b", // Packet Downlink Assignment (EGPRS)
+		"40284f0000001009810c826f4406809dcecb2b2b2b2b2b", // Packet Uplink Assignment (EGPRS)
+		"4024030f2f0000000087b0042b2b2b2b2b2b2b2b2b2b2b", // Packet Uplink Ack Nack (EGPRS)
+		"4913e00850884013a8048b2b2b2b2b2b2b2b2b2b2b2b2b", // Polling Request (malformed)
+		"412430007fffffffffffffffefd19c7ba12b2b2b2b2b2b", // Packet Uplink Ack Nack?
+		"41942b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b", // System Info 13?
+		"40883c1493120000000012002b2b2b2b2b2b2b2b2b2b2b", // Pkt Paging Request (OS#4838)
 	};
 
-	int testDataSize = sizeof(testData)/sizeof(testData[0]);
-
 	cout << " DOWNLINK " << endl;
-	for (int i = 0; i < testDataSize; i++)
+	for (unsigned int i = 0; i < ARRAY_SIZE(testData); i++)
 	{
 		cout << "vector1 = " << testData[i] << endl;
 		bitvec *vector = bitvec_alloc(23, test_ctx);
@@ -145,7 +84,7 @@ void testRlcMacDownlink(void *test_ctx)
 }
 
 
-void testRlcMacUplink(void *test_ctx)
+static void testRlcMacUplink(void *test_ctx)
 {
 	printf("*** %s ***\n", __func__);
 
@@ -154,25 +93,22 @@ void testRlcMacUplink(void *test_ctx)
 	bitvec_unhex(resultVector, DUMMY_VEC);
 
 	std::string testData[] = {
-	"400e1e61d11d2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b", // Packet Uplink Dummy Control Block
-	"400b8020000000000000002480e0032b2b2b2b2b2b2b2b", // Packet Downlink Ack/Nack
-	"4016713dc094270ca2ae57ef909006aa0fc0001f80222b", // Packet Resource Request
-	"401673c87f24af2632b25964200600000091000b780080", // Packet Resource Request (from keith)
-	"40200ffc0021ec010b2b2b2b2b2b2b2b2b2b2b2b2b2b2b", // EPDAN
-	"400a9020000000000000003010012a0800132b2b2b2b2b", // Packet Downlink Ack/Nack ?
+		"400e1e61d11d2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b", // Packet Uplink Dummy Control Block
+		"400b8020000000000000002480e0032b2b2b2b2b2b2b2b", // Packet Downlink Ack/Nack
+		"4016713dc094270ca2ae57ef909006aa0fc0001f80222b", // Packet Resource Request
+		"401673c87f24af2632b25964200600000091000b780080", // Packet Resource Request (from keith)
+		"40200ffc0021ec010b2b2b2b2b2b2b2b2b2b2b2b2b2b2b", // EPDAN
+		"400a9020000000000000003010012a0800132b2b2b2b2b", // Packet Downlink Ack/Nack ?
 
-	/* Packet Resource Request (see OS#4955, mistakes in 3GPP TS 44.060) */
-	"4117ea1b903eaeb2686564b2330820078000102b2b2b2b",
-	"40167e49f7f8ef2632b2596620060000806d00541c0080",
-	"4017787174d92eba686564b2ccc30800a000040b2b2b2b",
-	"4017e5b2cd5a2eca68655e44aec84880139300412b2b2b",
+		/* Packet Resource Request (see OS#4955, mistakes in 3GPP TS 44.060) */
+		"4117ea1b903eaeb2686564b2330820078000102b2b2b2b",
+		"40167e49f7f8ef2632b2596620060000806d00541c0080",
+		"4017787174d92eba686564b2ccc30800a000040b2b2b2b",
+		"4017e5b2cd5a2eca68655e44aec84880139300412b2b2b",
 	};
 
-	int testDataSize = sizeof(testData)/sizeof(testData[0]);
-
-
 	cout << " UPLINK " << endl;
-	for (int i = 0; i < testDataSize; i++)
+	for (unsigned int i = 0; i < ARRAY_SIZE(testData); i++)
 	{
 		cout << "vector1 = " << testData[i] << endl;
 		bitvec *vector = bitvec_alloc(23, test_ctx);
@@ -199,7 +135,7 @@ void testRlcMacUplink(void *test_ctx)
 	bitvec_free(resultVector);
 }
 
-void testCsnLeftAlignedVarBmpBounds(void *test_ctx)
+static void testCsnLeftAlignedVarBmpBounds(void *test_ctx)
 {
 	printf("*** %s ***\n", __func__);
 
@@ -227,7 +163,7 @@ extern "C" {
 int encode_gsm_ra_cap(struct bitvec *vector, MS_Radio_Access_capability_t * data);
 }
 
-void testRAcap(void *test_ctx)
+static void testRAcap(void *test_ctx)
 {
         printf("*** %s ***\n", __func__);
         MS_Radio_Access_capability_t data;
@@ -296,7 +232,7 @@ MS RA capability 1
         printf("decode_gsm_ra_cap() returns %d\n", rc);
 }
 
-void testMalformedRAcap(void *test_ctx)
+static void testMalformedRAcap(void *test_ctx)
 {
 	printf("*** %s ***\n", __func__);
 	MS_Radio_Access_capability_t data;
@@ -339,7 +275,7 @@ void testMalformedRAcap(void *test_ctx)
 }
 
 /* Reproduce crash from ticket OS#4463 */
-void testRAcap2(void *test_ctx)
+static void testRAcap2(void *test_ctx)
 {
 	printf("*** %s ***\n", __func__);
 	MS_Radio_Access_capability_t data;
@@ -480,7 +416,7 @@ MS Radio Access Capability
 }
 
 /* RAcap larger than 23 bytes */
-void testRAcap3(void *test_ctx)
+static void testRAcap3(void *test_ctx)
 {
 	printf("*** %s ***\n", __func__);
 	MS_Radio_Access_capability_t data;
@@ -677,7 +613,7 @@ MS Radio Access Capability
 }
 
 /* RAcap larger than 23 bytes */
-void testRAcap4(void *test_ctx)
+static void testRAcap4(void *test_ctx)
 {
 	printf("*** %s ***\n", __func__);
 	MS_Radio_Access_capability_t data;
@@ -818,7 +754,7 @@ MS Radio Access Capability
 	bitvec_free(bv_dec);
 }
 
-void testEGPRSPktChReq(void *test_ctx)
+static void testEGPRSPktChReq(void *test_ctx)
 {
 	EGPRS_PacketChannelRequest_t req;
 	int rc;
@@ -883,7 +819,6 @@ int main(int argc, char *argv[])
 	log_set_print_level(osmo_stderr_target, 1);
 	log_set_use_color(osmo_stderr_target, 0);
 
-	//printSizeofRLCMAC();
 	testRlcMacDownlink(ctx);
 	testRlcMacUplink(ctx);
 	testCsnLeftAlignedVarBmpBounds(ctx);
