@@ -30,6 +30,7 @@
 #include "decoding.h"
 #include "gprs_rlcmac.h"
 #include "egprs_rlc_compression.h"
+#include "alloc_algo.h"
 
 extern "C" {
 #include <osmocom/core/application.h>
@@ -687,7 +688,7 @@ static void test_egprs_ul_ack_nack()
 	bts->trx[0].pdch[4].enable();
 
 	GprsMs *ms = bts_alloc_ms(bts, 1, 1);
-	struct gprs_rlcmac_ul_tbf *tbf = tbf_alloc_ul_tbf(bts, ms, 0, true);
+	struct gprs_rlcmac_ul_tbf *tbf = ul_tbf_alloc(bts, ms, 0, true);
 	struct crbb_test crbb_test = {0};
 	bitvec *rbb = NULL;
 	unsigned int rbb_size;
@@ -779,9 +780,9 @@ void test_immediate_assign_dl()
 	bts->trx[0].pdch[3].enable();
 	GprsMs *ms = bts_alloc_ms(bts, 1, 0);
 
-	struct gprs_rlcmac_tbf *tbf = tbf_alloc_dl_tbf(bts, ms, 0, false);
+	struct gprs_rlcmac_tbf *tbf = dl_tbf_alloc(bts, ms, 0, false);
 	static uint8_t res[] = { 0x06,
-				 0x3f, /* Immediate Assignment Message Type */
+				 0x3f, /* Immediate Assignment Message Type (GSM48_MT_RR_IMM_ASS) */
 				 0x30, /* §10.5.2.26 Page Mode and §10.5.2.25b Dedicated mode/TBF */
 				 0x0d, 0x23, 0x6d, /* §10.5.2.25a Packet Channel Description */
 				 /* ETSI TS 44.018 §10.5.2.30 Request Reference */
@@ -804,9 +805,9 @@ void test_immediate_assign_ul0m()
 	bts->trx[0].pdch[5].enable();
 
 	GprsMs *ms = bts_alloc_ms(bts, 1, 0);
-	struct gprs_rlcmac_tbf *tbf = tbf_alloc_ul_tbf(bts, ms, 0, false);
+	struct gprs_rlcmac_tbf *tbf = ul_tbf_alloc(bts, ms, 0, false);
 	static uint8_t res[] = { 0x06,
-				 0x3f, /* Immediate Assignment Message Type */
+				 0x3f, /* Immediate Assignment Message Type (GSM48_MT_RR_IMM_ASS) */
 				 0x10, /* §10.5.2.26 Page Mode and §10.5.2.25b Dedicated mode/TBF */
 				 0x0d, 0x23, 0x6d, /* §10.5.2.25a Packet Channel Description */
 				 /* ETSI TS 44.018 §10.5.2.30 Request Reference */
@@ -824,7 +825,7 @@ void test_immediate_assign_ul0m()
 void test_immediate_assign_ul0s()
 {
 	static uint8_t res[] = { 0x06,
-				 0x3f, /* Immediate Assignment Message Type */
+				 0x3f, /* Immediate Assignment Message Type (GSM48_MT_RR_IMM_ASS) */
 				 0x10, /* §10.5.2.26 Page Mode and §10.5.2.25b Dedicated mode/TBF */
 				 0x0d, 0x23, 0x6d, /* §10.5.2.25a Packet Channel Description */
 				 /* ETSI TS 44.018 §10.5.2.30 Request Reference */
@@ -846,9 +847,9 @@ void test_immediate_assign_ul1s()
 	bts->trx[0].pdch[2].enable();
 
 	GprsMs *ms = bts_alloc_ms(bts, 1, 1);
-	struct gprs_rlcmac_tbf *tbf = tbf_alloc_ul_tbf(bts, ms, 0, false);
+	struct gprs_rlcmac_tbf *tbf = ul_tbf_alloc(bts, ms, 0, false);
 	static uint8_t res[] = { 0x06,
-				 0x3f, /* Immediate Assignment Message Type */
+				 0x3f, /* Immediate Assignment Message Type (GSM48_MT_RR_IMM_ASS) */
 				 0x10, /* §10.5.2.26 Page Mode and §10.5.2.25b Dedicated mode/TBF */
 				 0x0d, 0x23, 0x6d, /* §10.5.2.25a Packet Channel Description */
 				 /* ETSI TS 44.018 §10.5.2.30 Request Reference */
@@ -866,7 +867,7 @@ void test_immediate_assign_ul1s()
 void test_immediate_assign_ul1m()
 {
 	static uint8_t res[] = { 0x06,
-				 0x3f, /* Immediate Assignment Message Type */
+				 0x3f, /* Immediate Assignment Message Type (GSM48_MT_RR_IMM_ASS) */
 				 0x10, /* §10.5.2.26 Page Mode and §10.5.2.25b Dedicated mode/TBF */
 				 0x0d, 0x23, 0x6d, /* §10.5.2.25a Packet Channel Description */
 				 /* ETSI TS 44.018 §10.5.2.30 Request Reference */
